package provide DS9 1.0

proc ContourUpdate {} {
    global contour
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	if {$contour(view) && ([$current(frame) get fits count]>0)} {
	    ContourCheckParams
	    if {[winfo exist $contour(top)]} {
		# remove endl
		set levels ""
		regsub -all "\n" "[$contour(txt) get 1.0 end]" " " levels
		# and trim any trailing spaces
		set levels [string trimright $levels " "]
		if {$levels != ""} {
		    $current(frame) contour create $contour(color) \
			$contour(resolution) "\"$levels\""
		}
	    } else {
		$current(frame) contour create $contour(color) \
		    $contour(levels) $contour(resolution)
	    }
	} else {
	    $current(frame) contour delete
	}
	UpdateContourDialogMenu
	UnsetWatchCursor
    }
}

proc ContourCheckParams {} {
    global contour

    if {$contour(resolution) < 1} {
	set contour(resolution) 1
    }
    if {$contour(levels) < 1} {
	set contour(levels) 1
    }
}

proc ContourCopy {} {
    global contour
    global current

    set contour(copy) $current(frame)
    UpdateContourDialog
}

proc ContourPaste {sys} {
    global contour
    global current

    if {$current(frame) != "" && $contour(copy) != ""} {
	SetWatchCursor
	set pointer [$contour(copy) contour copy $sys]
	if {$pointer != 0} {
	    $current(frame) contour paste $contour(color) $pointer $sys
	}
	UnsetWatchCursor
	UpdateContourDialog
    }
}

proc ContourClear {} {
    global contour
    global current

    set contour(view) 0
    if {$current(frame) != ""} {
	$current(frame) contour delete
    }
    UpdateContourDialog
}

proc ContourDialog {} {
    global contour
    global current
    global menu
    global ds9

    # see if we already have a ctr window visible
    if [winfo exist $contour(top)] {
	return
    }

    set w $contour(top)
    set title "Contours"

    # create the contour window
    toplevel $w -colormap $ds9(main)
    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW ContourDestroyDialog

    # local variables
    set contour(txt) $w.param.right.level.value.text
    set txtscr $w.param.right.level.value.yscroll

    $w configure -menu $contour(mb)

    menu $contour(mb) -tearoff 0
    $contour(mb) add cascade -label File -menu $contour(mb).file
    $contour(mb) add cascade -label Edit -menu $contour(mb).edit
    $contour(mb) add cascade -label Color -menu $contour(mb).color
    $contour(mb) add cascade -label Scale -menu $contour(mb).scale

    menu $contour(mb).file -tearoff 0 -selectcolor $menu(selectcolor)
    $contour(mb).file add command -label "Apply" -command ContourApplyDialog
    $contour(mb).file add separator
    $contour(mb).file add cascade -label "Load Contours" \
	-menu $contour(mb).file.load
    $contour(mb).file add cascade -label "Save Contours" \
	-menu $contour(mb).file.save
    $contour(mb).file add separator
    $contour(mb).file add command -label "Load Contour Levels..." \
	-command ContourLoadLevels
    $contour(mb).file add command -label "Save Contour Levels..." \
	-command ContourSaveLevels
    $contour(mb).file add separator
    $contour(mb).file add command -label "Close" -command ContourDestroyDialog

    menu $contour(mb).file.load -tearoff 0 -selectcolor $menu(selectcolor)
    $contour(mb).file.load add command -label "Equatorial B1950" \
	-command {ContourLoad fk4}
    $contour(mb).file.load add command -label "Equatorial J2000" \
	-command {ContourLoad fk5}
    $contour(mb).file.load add command -label "ICRS" \
	-command {ContourLoad icrs}
    $contour(mb).file.load add command -label "Galactic" \
	-command {ContourLoad galactic}
    $contour(mb).file.load add command -label "Ecliptic" \
	-command {ContourLoad ecliptic}
    $contour(mb).file.load add separator
    $contour(mb).file.load add command -label "Linear" \
	-command {ContourLoad linear}
    $contour(mb).file.load add separator
    $contour(mb).file.load add command -label "Detector" \
	-command {ContourLoad detector}
    $contour(mb).file.load add command -label "Amplifier" \
	-command {ContourLoad amplifier}
    $contour(mb).file.load add command -label "Physical" \
	-command {ContourLoad physical}
    $contour(mb).file.load add command -label "Image" \
	-command {ContourLoad image}

    menu $contour(mb).file.save -tearoff 0 -selectcolor $menu(selectcolor)
    $contour(mb).file.save add command -label "Equatorial B1950" \
	-command {ContourSave fk4}
    $contour(mb).file.save add command -label "Equatorial J2000" \
	-command {ContourSave fk5}
    $contour(mb).file.save add command -label "ICRS" \
	-command {ContourSave icrs}
    $contour(mb).file.save add command -label "Galactic" \
	-command {ContourSave galactic}
    $contour(mb).file.save add command -label "Ecliptic" \
	-command {ContourSave ecliptic}
    $contour(mb).file.save add separator
    $contour(mb).file.save add command -label "Linear" \
	-command {ContourSave linear}
    $contour(mb).file.save add separator
    $contour(mb).file.save add command -label "Detector" \
	-command {ContourSave detector}
    $contour(mb).file.save add command -label "Amplifier" \
	-command {ContourSave amplifier}
    $contour(mb).file.save add command -label "Physical" \
	-command {ContourSave physical}
    $contour(mb).file.save add command -label "Image" \
	-command {ContourSave image}

    menu $contour(mb).edit -tearoff 0 -selectcolor $menu(selectcolor)
    $contour(mb).edit add command -label "Copy" -command ContourCopy
    $contour(mb).edit add cascade -label "Paste" -menu $contour(mb).edit.paste
    $contour(mb).edit add command -label "Clear" -command ContourClear

    menu $contour(mb).edit.paste -tearoff 0 -selectcolor $menu(selectcolor)
    $contour(mb).edit.paste add command -label "WCS" \
	-command {ContourPaste fk4}
    $contour(mb).edit.paste add command -label "Detector" \
	-command {ContourPaste detector}
    $contour(mb).edit.paste add command -label "Amplifier" \
	-command {ContourPaste amplifier}
    $contour(mb).edit.paste add command -label "Physical" \
	-command {ContourPaste physical}
    $contour(mb).edit.paste add command -label "Image" \
	-command {ContourPaste image}

    menu $contour(mb).color -tearoff 0 -selectcolor $menu(selectcolor)
    $contour(mb).color add radiobutton -label "Black" \
	-variable contour(color) -value black
    $contour(mb).color add radiobutton -label "White" \
	-variable contour(color) -value white
    $contour(mb).color add radiobutton -label "Red" \
	-variable contour(color) -value red
    $contour(mb).color add radiobutton -label "Green" \
	-variable contour(color) -value green
    $contour(mb).color add radiobutton -label "Blue" \
	-variable contour(color) -value blue
    $contour(mb).color add radiobutton -label "Cyan" \
	-variable contour(color) -value cyan
    $contour(mb).color add radiobutton -label "Magenta" \
	-variable contour(color) -value magenta
    $contour(mb).color add radiobutton -label "Yellow" \
	-variable contour(color) -value yellow

    menu $contour(mb).scale -tearoff 0 -selectcolor $menu(selectcolor)
    $contour(mb).scale add radiobutton -label Linear \
	-variable contour(scale) -value linear -command ContourUpdateLevels
    $contour(mb).scale add radiobutton -label Log \
	-variable contour(scale) -value log -command ContourUpdateLevels
    $contour(mb).scale add radiobutton -label Squared \
	-variable contour(scale) -value squared -command ContourUpdateLevels
    $contour(mb).scale add radiobutton -label "Square Root" \
	-variable contour(scale) -value sqrt -command ContourUpdateLevels

    set length 300

    frame $w.param
    frame $w.param.left
    frame $w.param.left.slide -relief groove -borderwidth 2
    frame $w.param.left.clip -relief groove -borderwidth 2
    frame $w.param.right
    frame $w.param.right.level -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2

    pack $w.param.left.slide $w.param.left.clip \
	-fill x -expand true -ipadx 4 -ipady 4
    pack $w.param.right.level -fill both -expand true
    pack $w.param.left $w.param.right -side left -fill both -expand true
    pack $w.param $w.buttons -fill x -expand true
    pack $w.buttons -ipadx 4 -ipady 4

    scale $w.param.left.slide.num -from 0 -to 50 -length $length \
	-variable contour(levels) -orient horizontal -label "Contour Levels" \
	-tickinterval 10 -showvalue true -resolution 1 \
	-command ContourUpdateLevelsScale
    scale $w.param.left.slide.res -from 0 -to 64 -length $length \
	-variable contour(resolution) -orient horizontal \
	-label "Contour Resolution" \
	-tickinterval 8 -showvalue true -resolution 1
    pack $w.param.left.slide.num $w.param.left.slide.res \
	-side top -ipadx 4 -ipady 4 -anchor w

    label $w.param.left.clip.title -text "Limits"

    label $w.param.left.clip.lowtitle -text "Low"
    entry $w.param.left.clip.low -textvariable contour(low) \
	-width 10 -relief sunken -bd 2

    label $w.param.left.clip.hightitle -text "High"
    entry $w.param.left.clip.high -textvariable contour(high) \
	-width 10 -relief sunken -bd 2

    grid $w.param.left.clip.title -row 0 -column 0 -padx 4 -sticky w
    grid $w.param.left.clip.lowtitle -row 0 -column 1 -padx 4 -sticky w
    grid $w.param.left.clip.low -row 0 -column 2 -padx 4 -sticky w
    grid $w.param.left.clip.hightitle -row 0 -column 3 -padx 4 -sticky w
    grid $w.param.left.clip.high -row 0 -column 4 -padx 4 -sticky w

    label $w.param.right.level.title -text "Levels"
    frame $w.param.right.level.value
    text $contour(txt) -height 13 -width 15 -wrap none \
	-font courier -yscrollcommand "$txtscr set"
    scrollbar $txtscr -command [list $contour(txt) yview] -orient vertical

    grid $w.param.right.level.value.text \
	$w.param.right.level.value.yscroll -sticky news
    grid $w.param.right.level.title -row 0 -column 0 -padx 4 -sticky w
    grid $w.param.right.level.value -row 1 -column 0 -padx 4 -sticky w

    button $w.buttons.apply -text "Apply" -command ContourApplyDialog
    button $w.buttons.close -text "Close" -command ContourDestroyDialog
    pack $w.buttons.apply $w.buttons.close -side left -padx 10 -expand true

    bind $w.param.left.clip.low <KeyRelease> ContourUpdateLevels
    bind $w.param.left.clip.high <KeyRelease> ContourUpdateLevels

    UpdateContourMenu
    UpdateContourDialog
}

proc ContourApplyDialog {} {
    global contour

    set contour(view) 1
    ContourUpdate
}

proc ContourDestroyDialog {} {
    global contour

    destroy $contour(top)
    destroy $contour(mb)
}

proc ContourUpdateLevelsScale {v} {
    ContourUpdateLevels
}

proc ContourUpdateLevels {} {
    global current
    global contour

    $contour(txt) delete 1.0 end
    if {($current(frame) != "") && 
	([$current(frame) get fits count]>0) &&
	($contour(low) != "") && 
	($contour(low) != "-") && 
	($contour(high) != "") &&
	($contour(high) != "-")} {

	    $contour(txt) insert end \
		[$current(frame) get contour level $contour(levels) \
		     $contour(low) $contour(high) $contour(scale)]
	}
}

proc UpdateContourMenu {} {
    global contour
    global current

    if {($current(frame) != "") && ([$current(frame) get fits count]>0)} {
	set contour(view) [$current(frame) has contour]

	set contour(scale) [$current(frame) get colorscale]
	set limits [$current(frame) get clip]
	set contour(low) [lindex $limits 0]
	set contour(high) [lindex $limits 1]
    } else {
	set contour(low) ""
	set contour(high) ""
    }
}

proc UpdateContourDialog {} {
    global contour
    global current

    if [winfo exist $contour(top)] {
	ContourUpdateLevels
	UpdateContourDialogMenu
    }
}

proc UpdateContourDialogMenu {} {
    global current
    global contour

    if [winfo exist $contour(top)] {
	UpdateCoordMenu $contour(mb).file.load 0 1 0
	UpdateCoordMenu $contour(mb).file.save 0 1 0

	if {$current(frame) != ""} {
	    if {[$current(frame) has contour]} {
		$contour(mb).edit entryconfig Copy -state normal
		$contour(mb).edit entryconfig Clear -state normal
	    } else {
		$contour(mb).edit entryconfig Copy -state disabled
		$contour(mb).edit entryconfig Clear -state disabled
	    }

	    if {$contour(copy) != ""} {
		if {[$contour(copy) has contour]} {
		    $contour(mb).edit entryconfig Paste -state normal

		    if {[$current(frame) has wcs equatorial] && 
			[$contour(copy) has wcs equatorial]} {
			$contour(mb).edit.paste entryconfig "WCS" \
			    -state normal
		    } elseif {[$current(frame) has wcs linear] && 
			[$contour(copy) has wcs linear]} {
			$contour(mb).edit.paste entryconfig "WCS" \
			    -state normal
		    } else {
			$contour(mb).edit.paste entryconfig "WCS" \
			    -state disabled
		    }

		    if {[$current(frame) has detector] && 
			[$contour(copy) has detector]} {
			$contour(mb).edit.paste entryconfig "Detector" \
			    -state normal
		    } else {
			$contour(mb).edit.paste entryconfig "Detector" \
			    -state disabled
		    }

		    if {[$current(frame) has amplifier] && 
			[$contour(copy) has amplifier]} {
			$contour(mb).edit.paste entryconfig "Amplifier" \
			    -state normal
		    } else {
			$contour(mb).edit.paste entryconfig "Amplifier" \
			    -state disabled
		    }

		    if {[$current(frame) has physical] &&
			[$contour(copy) has physical]} {
			$contour(mb).edit.paste entryconfig "Physical" \
			    -state normal
		    } else {
			$contour(mb).edit.paste entryconfig "Physical" \
			    -state disabled
		    }

		} else {
		    $contour(mb).edit entryconfig Paste -state disabled
		}
		
	    } else {
		$contour(mb).edit entryconfig Paste -state disabled
	    }
	}
    }
}

proc ContourLoadLevels {} {
    global contour
    global ds9

    set filename [OpenFileDialog $ds9(top) ds9(lastLevelName)]

    if {$filename != ""} {
	set id [open $filename r]
	$contour(txt) delete 1.0 end
	$contour(txt) insert end [read $id]
	close $id
    }
}

proc ContourSaveLevels {} {
    global contour
    global ds9

    set filename [SaveFileDialog $ds9(top) ds9(lastLevelName)]

    if {$filename != ""} {
	set id [open $filename w]
	puts -nonewline $id "[$contour(txt) get 1.0 end]"
	close $id
    }
}

proc ContourSave {sys} {
    global current
    global ds9

    set filename [SaveFileDialog $ds9(top) ds9(lastContourName)]

    if {$filename != ""} {
	if {$current(frame) != ""} {
	    $current(frame) contour save $filename $sys
	}
    }
}

proc ContourLoad {sys} {
    global current
    global contour
    global ds9

    set filename [OpenFileDialog $ds9(top) ds9(lastContourName)]

    if {$filename != ""} {
	if {$current(frame) != ""} {
	    SetWatchCursor
	    $current(frame) contour load $contour(color) $filename $sys
	    UnsetWatchCursor
	    UpdateContourDialog
	}
    }
}
