package provide DS9 1.0

# Modal Dialog Support Routines

proc DialogCreate {top title args} {
    global ds9

    eval {toplevel $top -colormap $ds9(main)} $args
    wm title $top $title
    wm iconname $top $title
}

proc DialogCenter {w} {
    # center dialog

    wm withdraw $w
    update idletasks
    set x [expr [winfo screenwidth $w]/2 - [winfo reqwidth $w]/2 \
	       - [winfo vrootx [winfo parent $w]]]
    set y [expr [winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
	       - [winfo vrooty [winfo parent $w]]]
    wm geom $w +$x+$y
    wm deiconify $w
}

proc DialogWait {top varName {focus {}}} {
    upvar $varName var

    if {[string length $focus] == 0} {
	set focus $top
    }
    set old [focus -displayof $top]
    focus $focus
    catch {tkwait visibility $top}
    catch {grab $top}
    tkwait variable $varName
    catch {grab release $top}
    focus $old
}

proc DialogDismiss {top} {
    # always destroy it
    destroy $top
}

# Simple List Box

proc SimpleListBox {w title sList} {
    global SLBReturn
    set SLBReturn 0

    SLBCreate $w $title $sList
    SLBWait $w

    return $SLBReturn
}

proc SLBCreate {w title sList} {
    global ds9
    global SLBReturn

    toplevel $w -colormap $ds9(main)
    wm title $w $title
    wm iconname $w $title
    wm transient $w $ds9(top)

    frame $w.buttons
    pack $w.buttons -side bottom -fill x
    button $w.buttons.ok -text OK -command [list SLBOK $w] -width 6
    button $w.buttons.cancel -text Cancel -command [list destroy $w] -width 6
    pack $w.buttons.ok $w.buttons.cancel -side left -expand yes \
	-padx 10 -pady 10 

    frame $w.frame
    pack $w.frame -side top -expand yes -fill y

    scrollbar $w.frame.scroll -command "$w.frame.list yview"
    listbox $w.frame.list -yscroll "$w.frame.scroll set" \
	-width 25 -height 15 -setgrid 1 -selectmode single
    pack $w.frame.list $w.frame.scroll -side left -fill y -expand 1

    foreach e $sList {
	$w.frame.list insert end $e
    }
    $w.frame.list selection set 0

    # center dialog

    wm withdraw $w
    update idletasks
    set x [expr [winfo screenwidth $w]/2 - [winfo reqwidth $w]/2 \
	       - [winfo vrootx [winfo parent $w]]]
    set y [expr [winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
	       - [winfo vrooty [winfo parent $w]]]
    wm geom $w +$x+$y
    wm deiconify $w

    bind $w <Double-1> [list SLBOK $w]
    bind $w <Return> [list SLBOK $w]

    bind $w <Up> [list SLBArrow $w.frame.list -1]
    bind $w <Down> [list SLBArrow $w.frame.list 1]
}

proc SLBWait {w} {
    set old [focus -displayof $w]
    focus $w
    catch {tkwait visibility $w}
    catch {grab $w}
    tkwait window $w
    catch {grab release $w}
    focus $old
}

proc SLBOK {w} {
    global SLBReturn

    set SLBReturn [expr [$w.frame.list curselection] +1]
    destroy $w
}

proc SLBArrow {w dir} {
    set which [$w curselection]
    set end [$w index end]

    $w selection clear 0 end
    incr which $dir
    if {$which < 0} {
	set which 0
    }
    if {$which >= $end} {
	set which [expr $end -1]
    }
    $w selection set $which
}

# Entry Dialog

proc EntryDialog {title message size varName} {
    upvar $varName var
    global ed

    set w ".ed"
    set ed(ok) 0
    set ed(value) $var

    DialogCreate $w $title -borderwidth 2
    frame $w.ed  -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.ed $w.buttons -fill x -ipadx 4 -ipady 4

    label $w.ed.title -text $message
    entry $w.ed.value -textvariable ed(value) -width $size \
	-relief sunken -bd 2
    pack $w.ed.title $w.ed.value -side top -padx 4 -pady 4
    pack $w.ed.title -anchor w
    pack $w.ed.value -ipadx 2 -ipady 2

    button $w.buttons.ok -text "OK" -default active \
	-command {set ed(ok) 1}
    button $w.buttons.cancel -text "Cancel" -command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    DialogCenter $w 
    $w.ed.value select range 0 end
    DialogWait $w ed(ok) $w.ed.value
    DialogDismiss $w

    if {$ed(ok)} {
	set var $ed(value)
    }
    
    return $ed(ok)
}

# Status Dialog

proc DisplayStatusMsg {w msg} {
    global ds9

    toplevel $w -colormap $ds9(main)
    wm protocol $w WM_DELETE_WINDOW DestroyStatusMsg

    frame $w.f -relief groove -borderwidth 2
    pack $w.f -fill x -padx 2 -pady 2 -ipadx 4 -ipady 4

    label $w.f.bitmap -bitmap "info"
    pack $w.f.bitmap -side left -padx 3m -pady 3m

    label $w.f.msg -justify left -text "$msg"
    catch {$w.f.msg configure -font \
	       -Adobe-Times-Medium-R-Normal--*-180-*-*-*-*-*-*
    }
    pack $w.f.msg -side right -expand 1 -fill both -padx 3m -pady 3m

    DialogCenter $w
    update idletasks
}

proc DestroyStatusMsg {w} {
    destroy $w
}

# Simple Text Dialog

proc SimpleTextDialog {varName width height title action pos} {
    upvar $varName var
    global ds9

    # see if we already have a window visible

    if {![winfo exist $var(top)]} {

	# create window

	toplevel $var(top) -colormap $ds9(main)
	wm title $var(top) $title
	wm iconname $var(top) $title
	wm group $var(top) $ds9(top)
	wm protocol $var(top) WM_DELETE_WINDOW \
	    "DestroySimpleTextDialog $varName"

	$var(top) configure -menu $var(mb)

	# simple menus

	menu $var(mb) -tearoff 0
	$var(mb) add cascade -label File -menu $var(mb).file
	$var(mb) add cascade -label Edit -menu $var(mb).edit

	menu $var(mb).file -tearoff 0
	$var(mb).file add command -label "Save..." \
	    -command "SaveSimpleTextDialog $varName"
	$var(mb).file add separator
	$var(mb).file add command -label "Print..." \
	    -command "PrintSimpleTextDialog $varName"
	$var(mb).file add separator
	$var(mb).file add command -label "Close" \
	    -command "DestroySimpleTextDialog $varName"

	menu $var(mb).edit -tearoff 0
	$var(mb).edit add command -label "Cut" -state disabled
	$var(mb).edit add command -label "Copy" -state disabled \
	    -command "CopySimpleTextDialog $varName"
	$var(mb).edit add command -label "Paste" -state disabled
	$var(mb).edit add command -label "Clear" -state disabled \
	    -command "ClearSimpleTextDialog $varName"
	$var(mb).edit add separator
	$var(mb).edit add command -label "Select All" \
	    -command "SelectAllSimpleTextDialog $varName"
	$var(mb).edit add command -label "Select None" \
	    -command "SelectNoneSimpleTextDialog $varName"
	$var(mb).edit add separator
	$var(mb).edit add command -label Find... \
	    -command "FindSimpleTextDialog $varName"
	$var(mb).edit add command -label "Find Next" -state disabled \
	    -command "FindNextSimpleTextDialog $varName"

	# create the text and scroll widgets
	
	set var(text) [text $var(top).text -height $height \
			   -width $width -wrap none \
			   -font courier \
			   -yscrollcommand "$var(top).yscroll set" \
			   -xscrollcommand "$var(top).xscroll set"]

	bind $var(top).text <ButtonRelease-1> \
	    "+ UpdateMenuSimpleTextDialog $varName"
	
	set sy [scrollbar $var(top).yscroll -command [list $var(text) yview] \
		    -orient vertical]
	set sx [scrollbar $var(top).xscroll -command [list $var(text) xview] \
		    -orient horizontal]
	grid $var(text) $sy -sticky news
	grid $sx -stick news
	grid rowconfigure $var(top) 0 -weight 1
	grid columnconfigure $var(top) 0 -weight 1
    }

    $var(text) configure -state normal
    if {$action != "append"} {
	$var(text) delete 1.0 end
    }
    $var(text) insert end "$var(txt)"
    switch $pos {
	top {$var(text) see 1.0}
	bottom {$var(text) see end}
    }
    $var(text) configure -state disabled
}

proc DestroySimpleTextDialog {varName} {
    upvar $varName var
    
    destroy $var(top)
    destroy $var(mb).file
    destroy $var(mb).edit
    destroy $var(mb)
}

proc UpdateMenuSimpleTextDialog {varName} {
    upvar $varName var

    if {[catch {selection get -displayof $var(top)}]} {
	$var(mb).edit entryconfig "Copy" -state disabled
	$var(mb).edit entryconfig "Clear" -state disabled
	$var(mb).edit entryconfig "Find Next" -state disabled
    } else {
	$var(mb).edit entryconfig "Copy" -state normal
	$var(mb).edit entryconfig "Clear" -state normal
	$var(mb).edit entryconfig "Find Next" -state normal
    }
}

proc CopySimpleTextDialog {varName} {
    upvar $varName var
    
    tk_textCopy $var(text)
}

proc ClearSimpleTextDialog {varName} {
    upvar $varName var
    
    $var(text) configure -state normal
    $var(text) delete sel.first sel.last
    UpdateMenuSimpleTextDialog var
    $var(text) configure -state disabled
}

proc SelectAllSimpleTextDialog {varName} {
    upvar $varName var

    $var(text) tag add sel 1.0 end
    UpdateMenuSimpleTextDialog var
}

proc SelectNoneSimpleTextDialog {varName} {
    upvar $varName var

    $var(text) tag remove sel 1.0 end
    UpdateMenuSimpleTextDialog var
}

proc FindSimpleTextDialog {varName} {
    upvar $varName var
    global ds9

    $var(text) tag remove sel 1.0 end
    set result "$ds9(lastTextSearch)"
    if {[EntryDialog "Search" "Enter Search Expression:" 40 result]} {
	set ds9(lastTextSearch) "$result"
	set start [$var(text) search -nocase -count cnt \
		       -regexp -- $result 1.0 end]
	if {$start != ""} {
	    $var(text) tag add sel $start "$start + $cnt chars"
	    $var(text) see $start
	}
    }
    UpdateMenuSimpleTextDialog var
}

proc FindNextSimpleTextDialog {varName} {
    upvar $varName var
    global ds9

    set result "$ds9(lastTextSearch)"
    set start [$var(text) search -nocase -count cnt \
		   -regexp -- $result sel.last end]
    if {$start != ""} {
	$var(text) tag remove sel 1.0 end
	$var(text) tag add sel $start "$start + $cnt chars"
	$var(text) see $start
    }
    UpdateMenuSimpleTextDialog var
}

proc PrintSimpleTextDialog {varName} {
    upvar $varName var
    global ps

    if {[PrintDialog txt]} { 
	if {$ps(dest) == "file"} {
	    catch {set ch [open "| cat > $ps(fileName)" w]}
	} else {
	    catch {set ch [open "| $ps(cmd)" w]}
	}

	if {$ch != ""} {
	    puts -nonewline $ch $var(txt)
	    close $ch
	} else {
	    Error "An error as occured while printing."
	    return
	}
    }
}

proc SaveSimpleTextDialog {varName} {
    upvar $varName var
    global ds9

    set fileName [SaveFileDialog $ds9(top) ds9(lastTextName)]
    if {$fileName != ""} {
	if {[catch {set ch [open "| cat > $fileName" w]}]} {
	    Error "An error as occured while saving."
	    return
	}
	puts -nonewline $ch $var(txt)
	close $ch
    }
}

