package provide DS9 1.0

proc DSSDialog {} {
    global dss
    global ds9
    global menu
    global current

    if [winfo exists $dss(top)] {
	return
    }

    set w $dss(top)
    set title "DSS Server"

    # create the window

    toplevel $w -colormap $ds9(main)
    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW DSSDestroyDialog

    # menu

    $w configure -menu $dss(mb)
    menu $dss(mb) -tearoff 0
    $dss(mb) add cascade -label File -menu $dss(mb).file
    $dss(mb) add cascade -label "DSS Server" -menu $dss(mb).server
    $dss(mb) add cascade -label "Name Server" -menu $dss(mb).name
    $dss(mb) add cascade -label Survey -menu $dss(mb).survey
    $dss(mb) add cascade -label Prefs -menu $dss(mb).prefs

    menu $dss(mb).file -tearoff 0 -selectcolor $menu(selectcolor)
    $dss(mb).file add command -label "Update from Current Frame" \
	-command UpdateDSSDialog
    $dss(mb).file add command -label Clear -command DSSClearDialog
    $dss(mb).file add separator
    $dss(mb).file add command -label Acknowledgement -command DSSAck
    $dss(mb).file add separator
    $dss(mb).file add command -label Close -command DSSDestroyDialog

    menu $dss(mb).server -tearoff 0 -selectcolor $menu(selectcolor)
    $dss(mb).server add radiobutton -label "DSS@SAO" \
	-variable dss(server) -value sao -command DSSServerDialog
    $dss(mb).server add radiobutton -label "DSS@STSCI" \
	-variable dss(server) -value stsci -command DSSServerDialog
    $dss(mb).server add radiobutton -label "DSS@ESO" \
	-variable dss(server) -value eso -command DSSServerDialog

    menu $dss(mb).name -tearoff 0 -selectcolor $menu(selectcolor)
    $dss(mb).name add radiobutton -label "NED@SAO" \
	-variable nsvr(server) -value ned-sao
    $dss(mb).name add radiobutton -label "NED@ESO" \
	-variable nsvr(server) -value ned-eso
    $dss(mb).name add radiobutton -label "SIMBAD@SAO" \
	-variable nsvr(server) -value simbad-sao
    $dss(mb).name add radiobutton -label "SIMBAD@ESO" \
	-variable nsvr(server) -value simbad-eso

    menu $dss(mb).coord -tearoff 0 -selectcolor $menu(selectcolor)
    $dss(mb).coord add radiobutton -label "Equatorial B1950" \
	-variable dss(system) -value fk4 -command DSSCoordDialog
    $dss(mb).coord add radiobutton -label "Equatorial J2000" \
	-variable dss(system) -value fk5 -command DSSCoordDialog
    $dss(mb).coord add radiobutton -label "ICRS" \
	-variable dss(system) -value icrs -command DSSCoordDialog
    $dss(mb).coord add radiobutton -label "Galactic" \
	-variable dss(system) -value galactic -command DSSCoordDialog
    $dss(mb).coord add radiobutton -label "Ecliptic" \
	-variable dss(system) -value ecliptic -command DSSCoordDialog

    menu $dss(mb).survey -tearoff 0 -selectcolor $menu(selectcolor)
    $dss(mb).survey add radiobutton -label "DSS1" \
	-variable dss(survey) -value dss
    $dss(mb).survey add radiobutton -label "DSS2-red" \
	-variable dss(survey) -value dss2red
    $dss(mb).survey add radiobutton -label "DSS2-blue" \
	-variable dss(survey) -value dss2blue

    menu $dss(mb).prefs -tearoff 0 -selectcolor $menu(selectcolor)
    $dss(mb).prefs add checkbutton -label "Save FITS on download" \
	-variable dss(save)
    $dss(mb).prefs add checkbutton -label "Set size from Current Image" \
	-variable dss(updatesize)
    $dss(mb).prefs add separator
    $dss(mb).prefs add radiobutton -label "New Frame" \
	-variable dss(frame) -value new
    $dss(mb).prefs add radiobutton -label "Current Frame" \
	-variable dss(frame) -value current

    frame $w.param -relief groove -borderwidth 2
    frame $w.status -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.param $w.status $w.buttons -fill x -expand true -ipadx 4 -ipady 4

    label $w.param.nametitle -text "Name"
    entry $w.param.name -textvariable dss(name) -width 38 -relief sunken -bd 2

    set dss(xname) [label $w.param.xtitle -text "" -font symbol -width 1]
    entry $w.param.x -textvariable dss(x) -width 14 -relief sunken -bd 2
    set dss(yname) [label $w.param.ytitle -text "" -font symbol -width 1]
    entry $w.param.y -textvariable dss(y) -width 14 -relief sunken -bd 2
    label $w.param.system -textvariable dss(system) \
	-width 10 -relief groove -bd 2

    label $w.param.wtitle -text "Width"
    entry $w.param.w -textvariable dss(width) -width 14 -relief sunken -bd 2
    label $w.param.htitle -text "Height"
    entry $w.param.h -textvariable dss(height) -width 14 -relief sunken -bd 2
    label $w.param.format -textvariable dss(format) \
	-width 10 -relief groove -bd 2

    grid rowconfigure $w.param 0 -pad 4
    grid rowconfigure $w.param 1 -pad 4
    grid rowconfigure $w.param 2 -pad 4

    grid $w.param.nametitle -row 0 -column 0 -padx 4 -sticky w
    grid $w.param.name -row 0 -column 1 -columnspan 4 -padx 4 -sticky w

    grid $w.param.xtitle -row 1 -column 0 -padx 4 -sticky w
    grid $w.param.x -row 1 -column 1 -padx 4 -sticky w
    grid $w.param.ytitle -row 1 -column 2 -padx 4 -sticky w
    grid $w.param.y -row 1 -column 3 -padx 4 -sticky w
    grid $w.param.system -row 1 -column 4 -padx 4 -sticky w

    grid $w.param.wtitle -row 2 -column 0 -padx 4 -sticky w
    grid $w.param.w -row 2 -column 1 -padx 4 -sticky w
    grid $w.param.htitle -row 2 -column 2 -padx 4 -sticky w
    grid $w.param.h -row 2 -column 3 -padx 4 -sticky w
    grid $w.param.format -row 2 -column 4 -padx 4 -sticky w

    label $w.status.item -textvariable dss(status)
    pack $w.status.item -anchor w -pady 4

    set dss(apply) \
	[button $w.buttons.apply -text "Retrieve" -command DSSApplyDialog]
    set dss(cancel) \
	[button $w.buttons.cancel -text "Cancel" -command DSSCancelDialog \
	     -state disabled]
    button $w.buttons.close -text "Close" -command DSSDestroyDialog
    pack $w.buttons.apply $w.buttons.cancel $w.buttons.close \
	-side left -padx 10 -expand true

    UpdateDSSDialog
    DSSCoordDialog
    DSSServerDialog
    DSSStatus {}
}

proc DSSDestroyDialog {} {
    global dss

    if {[info exists dss(token)]} {
	http::reset $dss(token)
    }
    destroy $dss(top)
    destroy $dss(mb)
}

proc DSSApplyDialog {} {
    global dss
    global nsvr
    global ds9

    DSSStatus {}
    $dss(apply) configure -state disabled
    $dss(cancel) configure -state normal

    if {$dss(name) != ""} {
	DSSStatus "Looking up $dss(name)"
	set dss(x) {}
	set dss(y) {}
	set coord [NameSVR $nsvr(server) $dss(name)]
	set dss(x) [lindex $coord 0]
	set dss(y) [lindex $coord 1]
	set dss(system) fk5
	DSSCoordDialog

	if {($dss(x) == "") || ($dss(y) == "")} {
	    set dss(x) {}
	    set dss(y) {}
	    DSSStatus "$dss(name) not found"

	    $dss(apply) configure -state normal
	    $dss(cancel) configure -state disabled
	    return
	}
    }

    DSSStatus "Contacting DSS Image Server"
    if {($dss(x) != "") && ($dss(y) != "")} {
	switch $dss(frame) {
	    new {
		set ds9(displaymode) tile
		CreateFrame
	    }
	    current {}
	}

	switch $dss(server) {
	    sao {DSSSAO}
	    stsci {DSSSTSCI}
	    eso {DSSESO}
	}
    }    

    DSSStatus {Done}
    $dss(apply) configure -state normal
    $dss(cancel) configure -state disabled
}

proc DSSCancelDialog {} {
    global dss

    if {[info exists dss(token)]} {
	http::reset $dss(token)
    }

    $dss(apply) configure -state normal
    $dss(cancel) configure -state disabled
}

proc DSSStatus {message} {
    global dss

    set dss(status) "Status: $message"
}

proc DSSProgress {token totalsize currentsize} {
    if {$totalsize != 0} {
	DSSStatus "$currentsize bytes of $totalsize bytes [expr int(double($currentsize)/$totalsize*100)]%"
    } else {
	DSSStatus "$currentsize bytes"
    }
}

proc DSSClearDialog {} {
    global dss

    set dss(name) ""
    set dss(x) ""
    set dss(y) ""
    DSSStatus {}
}

proc DSSServerDialog {} {
    global dss

    switch $dss(server) {
	sao {$dss(mb) entryconfig "Survey" -state disabled}
	stsci {$dss(mb) entryconfig "Survey" -state normal}
	eso {$dss(mb) entryconfig "Survey" -state normal}
    }
}

proc DSSCoordDialog {} {
    global dss

    switch $dss(system) {
	fk4 -
	fk5 -
	icrs {
	    $dss(xname) configure -text "a" -font symbol
	    $dss(yname) configure -text "d" -font symbol
	}
	galactic {
	    $dss(xname) configure -text "l" -font {-family times -slant italic}
	    $dss(yname) configure -text "b" -font {-family times -slant italic}
	}
	ecliptic {
	    $dss(xname) configure -text "l" -font symbol
	    $dss(yname) configure -text "b" -font symbol
	}
    }
}

proc DSSSAO {} {
    global dss
    global ds9

    if {$dss(save)} {
	set compress no
	set fn [SaveFileDialog $ds9(top) ds9(lastDSSName)]
	if {$fn == ""} {
	    return
	}

    } else {
	set compress gzip
	set fn "/tmp/dss.fits.gz"
    }

    set query [http::formatQuery r $dss(x) d $dss(y) e J2000 \
		   w $dss(width) h $dss(height) \
		   c $compress]

    # Load image

    LoadHTTP "http://archive.harvard.edu/archive/dss" \
	"$fn" $dss(save) {} DSSProgress $query
}

proc DSSESO {} {
    global dss
    global ds9

    switch $dss(survey) {
	dss {set survey "DSS1"}
	dss2red {set survey "DSS2-red"}
	dss2blue {set survey "DSS2-blue"}
    }

    if {$dss(save)} {
	set mime "application/x-fits"
	set fn [SaveFileDialog $ds9(top) ds9(lastDSSName)]
	if {$fn == ""} {
	    return
	}
    } else {
	set mime "display/gz-fits"
	set fn "/tmp/dss.fits.gz"
    }

    set query [http::formatQuery ra $dss(x) dec $dss(y) equinox J2000\
		   x $dss(width) y $dss(height) \
		   mime-type "$mime" \
		   Sky-Survey $survey]

    # Load image
    # we can't use -query because eso needs a GET not a POST

    LoadHTTP "http://archive.eso.org/dss/dss?$query" \
	"$fn" $dss(save) {} DSSProgress {}
}

proc DSSSTSCI {} {
    global dss
    global ds9

    switch $dss(survey) {
	dss {set survey "1"}
	dss2red {set survey "2r"}
	dss2blue {set survey "2b"}
    }

    if {$dss(save)} {
	set compress none
	set fn [SaveFileDialog $ds9(top) ds9(lastDSSName)]
	if {$fn == ""} {
	    return
	}

    } else {
	set compress gz
	set fn "/tmp/dss.fits.gz"
    }

    set query [http::formatQuery r $dss(x) d $dss(y) e J2000\
		   w $dss(width) h $dss(height) \
		   f fits c $compress \
		   v $survey]

    # Load image

    LoadHTTP "http://stdatu.stsci.edu/cgi-bin/dss_search" \
	"$fn" $dss(save) {} DSSProgress $query
}

proc UpdateDSSDialog {} {
    global current
    global dss

    if {[winfo exist $dss(top)]} {
	if {$current(frame) != ""} {
	    if {[$current(frame) has wcs equatorial]} {
		set coord [$current(frame) get cursor $dss(system) sexagesimal]
		set dss(x) [lindex $coord 0]
		set dss(y) [lindex $coord 1]

		if {$dss(updatesize)} {
		    set size \
			[$current(frame) get fits size $dss(system) arcmin]
		    set dss(width) [lindex $size 0]
		    set dss(height) [lindex $size 1]
		}
	    }
	}
    }
}

proc DSSAck {} {
    global ackDialog

set msg {Acknowledgements for the DSS 

If the image is from the northern plates (POSS):  Based on photographic data
of the National Geographic Society -- Palomar Geographic Society to the
California Institute of Technology. The plates were processed into the present
compressed digital form with their permission.  The Digitized Sky Survey was
produced at the Space Telescope Science Institute under US Government grant
NAG W-2166. Investigators using these scans are requested to include the above
acknowledgements in any publications.

Copyright (c) 1994, Association of Universities for Research in Astronomy,
Inc. All rights reserved.

If the image is from the southern plates (SERC-J):  Based on photographic data
obtained using The UK Schmidt Telescope. The UK Schmidt Telescope was operated
by the Royal Observatory Edinburgh, with funding from the UK Science and
Engineering Research Council, until 1988 June, and thereafter by the
Anglo-Australian Observatory. Original plate material is copyright (c) the
Royal Observatory Edinburgh and the Anglo-Australian Observatory. The plates
were processed into the present compressed digital form with their permission.
The Digitized Sky Survey was produced at the Space Telescope Science Institute
under US Government grant NAG W-2166. Investigators using these scans are
requested to include the above acknowledgements in any publications.

Copyright (c) 1993, 1994, Association of Universities for Research in
Astronomy, Inc. All rights reserved.  
}

    set ackDialog(top) .ack
    set ackDialog(mb) .ackMB
    set ackDialog(txt) "$msg"

    SimpleTextDialog ackDialog 80 40 "Acknowledgement" insert top
}
