package provide DS9 1.0

proc RectangleDialog {frame id} {
    global marker

    # see if we already have a header window visible

    set w ".marker$id"
    set mb ".mb$id"

    if [winfo exist $w] {
	return
    }

    # Init common items

    InitMarkerDialog $frame $id
    PropMenuMarkerDialog $frame $mb $id
    CoordMenuMarkerDialog $frame $mb $id CoordRectangleCB

    # rectangle menu items

    set marker($frame,$id,sizeformat) pixels
    DistMenuMarkerDialog $frame $mb $id CoordRectangleCB sizeformat
    $mb add cascade -label Size -menu $mb.sizeformat

    # Init rectangle variables

    EditRectangleCB $frame $id
    RotateRectangleCB $frame $id

    # rectangle specific callbacks

    $frame marker $id callback edit EditRectangleCB $frame
    $frame marker $id callback rotate RotateRectangleCB $frame
    $frame marker $id callback delete DeleteRectangleDialog $frame

    # Dialog

    CommonMarkerDialog $frame $id
    wm protocol $w WM_DELETE_WINDOW "CloseRectangleDialog $frame $id"

    # Size

    label $w.ref.widthTitle -text "Width/Height"
    entry $w.ref.width -textvariable marker($frame,$id,width) \
	-width 13 -relief sunken -bd 2
    entry $w.ref.height -textvariable marker($frame,$id,height) \
	-width 13 -relief sunken -bd 2
    label $w.ref.sizeCoord -textvariable marker($frame,$id,sizeformat) \
	-relief groove -bd 2 -width 9 -padx 4 -text "pixels"
    label $w.ref.angleTitle -text "Angle"
    entry $w.ref.angle -textvariable marker($frame,$id,angle) \
	-width 13 -relief sunken -bd 2
    label $w.ref.angleUnit -relief groove -bd 2 -width 9 -padx 4 -text "degrees"

    grid $w.ref.widthTitle -row 1 -column 0 -padx 4 -sticky w
    grid $w.ref.width -row 1 -column 1 -padx 4 -sticky w
    grid $w.ref.height -row 1 -column 2 -padx 4 -sticky w
    grid $w.ref.sizeCoord -row 1 -column 3 -padx 4 -sticky w

    grid $w.ref.angleTitle -row 2 -column 0 -padx 4 -sticky w
    grid $w.ref.angle -row 2 -column 1 -padx 4 -sticky w
    grid $w.ref.angleUnit -row 2 -column 3 -padx 4 -sticky w

    # Buttons

    button $w.buttons.apply -text "Apply" \
	-command "ApplyRectangleDialog $frame $id"
    button $w.buttons.close -text "Close" \
	-command "CloseRectangleDialog $frame $id"
    pack $w.buttons.apply $w.buttons.close -side left -padx 10 -expand true

    bind $w <Return> "ApplyRectangleDialog $frame $id"
}

proc ApplyRectangleDialog {frame id} {
    EditRectangleDialog $frame $id
    RotateRectangleDialog $frame $id
    ApplyMarkerDialog $frame $id
}

proc CloseRectangleDialog {frame id} {
    global marker

    unset marker($frame,$id,width)
    unset marker($frame,$id,height)
    unset marker($frame,$id,angle)

    $frame marker $id delete callback edit EditRectangleCB
    $frame marker $id delete callback rotate RotateRectangleCB
    $frame marker $id delete callback delete DeleteRectangleDialog
    DeleteMarkerCBs $frame $id

    DeleteMarkerDialog $frame $id
}

proc DeleteRectangleDialog {frame id} {
    global marker

    unset marker($frame,$id,width)
    unset marker($frame,$id,height)
    unset marker($frame,$id,angle)
    unset marker($frame,$id,sizeformat)

    set mb ".mb$id"
    destroy $mb.sizeformat

    DeleteMarkerDialog $frame $id
}

proc EditRectangleDialog {frame id} {
    global marker

    $frame marker $id rectangle size \
	$marker($frame,$id,width) $marker($frame,$id,height) \
	$marker($frame,$id,coord) $marker($frame,$id,sizeformat)
}

proc RotateRectangleDialog {frame id} {
    global marker

    $frame marker $id rectangle angle $marker($frame,$id,angle)
}

proc EditRectangleCB {frame id} {
    global marker

    set f [$frame get marker $id rectangle size \
	       $marker($frame,$id,coord) $marker($frame,$id,sizeformat)]

    set marker($frame,$id,width) [lindex $f 0]
    set marker($frame,$id,height) [lindex $f 1]
}

proc RotateRectangleCB {frame id} {
    global marker

    set marker($frame,$id,angle) [$frame get marker $id rectangle angle]
}

proc CoordRectangleCB {frame id} {
    MoveMarkerCB $frame $id
    EditRectangleCB $frame $id
}

proc RectangleDefaultDialog {} {
    global rectangle
    global ed

    set w ".rectangle"
    set title "Default Rectangle"
    set ed(ok) 0
    set ed(width) $rectangle(width)
    set ed(height) $rectangle(height)

    DialogCreate $w $title -borderwidth 2
    frame $w.ed  -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.ed $w.buttons -fill x -ipadx 4 -ipady 4

    label $w.ed.title -text "Width"
    entry $w.ed.width -textvariable ed(width) -width 10 \
	-relief sunken -bd 2
    label $w.ed.unit -text "pixels" -relief groove -width 8
    
    label $w.ed.title2 -text "Height"
    entry $w.ed.height -textvariable ed(height) -width 10 \
	-relief sunken -bd 2
    label $w.ed.unit2 -text "pixels" -relief groove -width 8

    grid $w.ed.title -row 1 -column 0 -padx 4 -sticky w
    grid $w.ed.width -row 1 -column 1 -padx 4 -sticky w
    grid $w.ed.unit -row 1 -column 2 -padx 4 -sticky w
    grid $w.ed.title2 -row 2 -column 0 -padx 4 -sticky w
    grid $w.ed.height -row 2 -column 1 -padx 4 -sticky w
    grid $w.ed.unit2 -row 2 -column 2 -padx 4 -sticky w

    button $w.buttons.ok -text "OK" -default active \
	-command {set ed(ok) 1}
    button $w.buttons.cancel -text "Cancel" -command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    DialogCenter $w 
    $w.ed.width select range 0 end
    DialogWait $w ed(ok) $w.ed.width
    DialogDismiss $w

    if {$ed(ok)} {
	set rectangle(width) $ed(width)
	set rectangle(height) $ed(height)
    }
    unset ed(width)
    unset ed(height)
}
