%option noyywrap
%option caseless
%option never-interactive
%option c++

%{
  #include <stdio.h>
  #include <stdlib.h>
  #include <string.h>

  #include "parser.C.h"

%}

%x EXT

D   [0-9]
E   [Ee][+-]?{D}+

/* rules */

%%

[!-@A-Z_-~]+	{ // File or Column Name
		  strcpy(fflval.str,yytext);
		  return STRING;
		}

[ \t]+		{ // White Spaces
		} 

\[		{ // Start to process options
		  BEGIN EXT;
		  return yytext[0];
		}

.		{ // Else, return the char
		  return yytext[0];
		}

<EXT>{

arch		{return ARCH_;}
bigendian	{return BIGENDIAN_;}
bin		{return BIN_;}
binkey		{return BINKEY_;}
bitpix		{return BITPIX_;}
dim		{return DIM_;}
dims		{return DIMS_;}
key		{return KEY_;}
littleendian	{return LITTLEENDIAN_;}
skip		{return SKIP_;}
xdim		{return XDIM_;}
ydim		{return YDIM_;}

[+-]?{D}+	{ // Integer
		  fflval.integer = atoi(yytext);
		  return INT;
		}

[+-]?{D}+"."?({E})? |
[+-]?{D}*"."{D}+({E})? { // Real Number
		  fflval.real = atof(yytext);
		  return REAL;
		}

[0-9A-Za-z_]+	{ // General String
		  strcpy(fflval.str,yytext);
		  return STRING;
		}

[ \t]+		{ // White Spaces
		} 

.		{ // Else, return the char
		  return yytext[0];
		}
}

%%






