#include "annuli.h"
#include "frame.h"
#include "util.h"

// Annuli Markers Public

AnnuliMarker::AnnuliMarker(Frame* p, const Vector& ctr, int s, const char* clr, 
			   const char* f, const char* t, unsigned short prop)
  : segments(s), Marker(p, ctr, clr, f, t, prop)
{
  annuli = 1;
  ratio[0] = 1.0;
  vertices = NULL;
}

void AnnuliMarker::updateBBox()
{
  // bound marker

  newVertices();
  calcPoints(parent->getZoom());

  Matrix& m = parent->refToCanvas;
  bbox = BBox(vertices[0][0] * m, vertices[0][0] * m); // init the bbox
  for (int i=0; i<annuli; i++)
    for (int j=0; j<segments; j++)
      bbox.bound(vertices[i][j] * m);

  deleteVertices();

  // generate handles

  updateHandles();

  // bound handles

  {
    for (int i=0; i<numHandle; i++)
      bbox.bound(handle[i]);
  }

  // make room for handles

  bbox.expand(3);

  // calculate overall bbox

  calcAllBBox();
}

void AnnuliMarker::draw()
{
  Marker::draw();
  includeExclude();
  render(parent->pixmap, parent->refToWidget, gc, parent->getZoom());
}

void AnnuliMarker::drawXOR()
{
  render(Tk_WindowId(parent->tkwin), parent->refToWindow, gcxor, parent->getZoom());
}

void AnnuliMarker::drawMagnifier(const Matrix& m)
{
  XSetForeground(display, gc, color);
  includeExclude();
  render(parent->magnifierPixmap, m, gc, parent->getMagnifierZoom());
}

void AnnuliMarker::ps(int mode)
{
  Marker::ps(mode);
  psIncludeExclude();

  newVertices();
  calcPoints(parent->getZoom());

  char buf[256];
  for (int i=0; i<annuli; i++) {
    for (int j=0; j<segments; j++) {
      ostrstream str(buf,256);
      Vector v =  vertices[i][j] * parent->refToCanvas;
      if (j==0)
	str << "newpath " << endl
	    << v.TkCanvasPs(parent->canvas) << " moveto" << endl << ends;
      else
	str << v.TkCanvasPs(parent->canvas) << " lineto" << endl << ends;

      Tcl_AppendResult(parent->interp, buf, NULL);
    }
    ostrstream str(buf,256);
    str << "closepath stroke" << endl << ends;
    Tcl_AppendResult(parent->interp, buf, NULL);
  }

  deleteVertices();
}

void AnnuliMarker::addAnnuli(double r)
{
  if (annuli<MAXANNULI) {
    ratio[annuli] = r;
    annuli++;
  }
  else
    cerr << "Marker Internal Error: maxium number of annuli exceeded" << endl;
}

// Annuli Protected

double AnnuliMarker::maxAnnuli()
{
  double max = 0;
  for (int i=0; i<annuli; i++)
    if (max < ratio[i])
      max = ratio[i];
  return max;
}

// Annuli Markers Private

void AnnuliMarker::render(Drawable drawable, const Matrix& matrix, GC ggc, 
			  double zoom)
{
  newVertices();
  calcPoints(zoom);

  int count = segments+1;
  for (int i=0; i<annuli; i++) {
    XPoint* p = new XPoint[count];
    for (int j=0; j<segments; j++) {
      Vector v = (vertices[i][j] * matrix).round();
      p[j].x = (int)v[0];
      p[j].y = (int)v[1];
    }
    p[segments] = p[0];
    XDrawLines(display, drawable, ggc, p, count, CoordModeOrigin);
    delete p;
  }

  deleteVertices();
}

void AnnuliMarker::newVertices() 
{
  if (vertices)
    deleteVertices();

  vertices = new Vector*[annuli];
  for (int i=0; i<annuli; i++)
    vertices[i]= new Vector[segments];
}

void AnnuliMarker::deleteVertices()
{
  for (int i=0; i<annuli; i++)
    delete [] vertices[i];
  delete [] vertices;
  vertices = NULL;
}
