#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "colorscale.h"
#include "util.h"

ColorScale::ColorScale(int s)
{
  size_ = s;
  psColors_ = new unsigned char[s*3];
  psIndex_ = new unsigned short[s];

  memset(psColors_, '0', size_*3);
  memset(psIndex_, '0', size_*sizeof(unsigned short));
}

ColorScale::~ColorScale()
{
  if (psColors_)
    delete psColors_;

  if (psIndex_)
    delete psIndex_;

  if (colors_)
    delete [] colors_;
}

LinearScale::LinearScale(int s, unsigned short* indexCells, 
			 unsigned char* colorCells, int count) : ColorScale(s)
{
  for (int i=0; i<s; i++) {
    int l = (int)((double)i/s * count);
    psIndex_[i] = indexCells[l];
    memcpy(psColors_+i*3, colorCells+l*3,3);
  }
}

/* no longer used
 * this algorithm has some series problems.
LnScale::LnScale(int s, unsigned index* indexCells, unsigned char* colorCells,
		 int count) : ColorScale(s)
{
  // y = log((x/xSize*(e**10-1)+1)/10 * ySize
  // This algorthim has the following properties:
  // 1. y range log curve from 0 to 1
  // 2. x range independent

  double exp1 = exp(10.)-1;
  for (int i=0; i<s; i++) {
    int l = (int)(log(((double)i/s*exp1)+1)/10 * count);
    psIndex_[i] = indexCells[l];
    memcpy(psColors_+i*3, colorCells+l*3,3);
  }
}
*/

LogScale::LogScale(int s, unsigned short* indexCells, 
		   unsigned char* colorCells, int count) : ColorScale(s)
{
  for (int i=0; i<s; i++) {
    int l;
    if (i>0)
      l = (int)((log10(i)/log10(s))*count);
    else
      l = 0;
    psIndex_[i] = indexCells[l];
    memcpy(psColors_+i*3, colorCells+l*3,3);
  }
}

SquaredScale::SquaredScale(int s, unsigned short* indexCells, 
			   unsigned char* colorCells, int count) 
  : ColorScale(s)
{
  for(int i=0; i<s; i++) {
    double a = (double)i/s;
    int l = (int)(a*a * count);
    psIndex_[i] = indexCells[l];
    memcpy(psColors_+i*3, colorCells+l*3,3);
  }
}

SqrtScale::SqrtScale(int s, unsigned short* indexCells, 
		     unsigned char* colorCells, int count) : ColorScale(s)
{
  for(int i=0; i<s; i++) {
    int l = (int)(sqrt((double)i/s) * count);
    psIndex_[i] = indexCells[l];
    memcpy(psColors_+i*3, colorCells+l*3,3);
  }
}

IISScale::IISScale(unsigned short* indexCells, 
		   unsigned char* colorCells, int count) : ColorScale(IISSIZE)
{
  {
    for (int i=0; i<IISCOLORS; i++) {
      int l = (int)((double)i/IISCOLORS * count);
      psIndex_[i] = indexCells[l];
      memcpy(psColors_+i*3, colorCells+l*3,3);
    }
  }

  {
    for (int i=IISCOLORS; i<IISSIZE; i++)
      psIndex_[i] = i;
  }

  setRGBColor(psColors_+200*3,"black");
  setRGBColor(psColors_+201*3,"black");
  setRGBColor(psColors_+202*3,"white");
  setRGBColor(psColors_+203*3,"red");
  setRGBColor(psColors_+204*3,"green");
  setRGBColor(psColors_+205*3,"blue");
  setRGBColor(psColors_+206*3,"yellow");
  setRGBColor(psColors_+207*3,"cyan");
  setRGBColor(psColors_+208*3,"magenta");
}

void IISScale::setRGBColor(unsigned char* ptr, const char* colorName)
{
  if (!strcmp(colorName,"black")) {
    *(ptr+0) = 000;
    *(ptr+1) = 000;
    *(ptr+2) = 000;
  }
  else if (!strcmp(colorName,"white")) {
    *(ptr+0) = 255;
    *(ptr+1) = 255;
    *(ptr+2) = 255;
  }
  else if (!strcmp(colorName,"red")) {
    *(ptr+0) = 000;
    *(ptr+1) = 000;
    *(ptr+2) = 255;
  }
  else if (!strcmp(colorName,"green")) {
    *(ptr+0) = 000;
    *(ptr+1) = 255;
    *(ptr+2) = 000;
  }
  else if (!strcmp(colorName,"blue")) {
    *(ptr+0) = 255;
    *(ptr+1) = 000;
    *(ptr+2) = 000;
  }
  else if (!strcmp(colorName,"cyan")) {
    *(ptr+0) = 255;
    *(ptr+1) = 255;
    *(ptr+2) = 000;
  }
  else if (!strcmp(colorName,"magenta")) {
    *(ptr+0) = 255;
    *(ptr+1) = 000;
    *(ptr+2) = 255;
  }
  else if (!strcmp(colorName,"yellow")) {
    *(ptr+0) = 000;
    *(ptr+1) = 255;
    *(ptr+2) = 255;
  }
  else {
    cerr << "setRGBColor Internal Error: Unable to Parser color: "
	 << colorName << endl;
    *(ptr+0) = 000;
    *(ptr+1) = 000;
    *(ptr+2) = 000;
  }
}
